/* libmb - A library of utility functions used by matchbox utilitys
 * Copyright (C) 2003 Matthew Allum
 *
 * SPDX-License-Identifier: LGPL-2.0-or-later
 */

#ifndef _MB_H_
#define _MB_H_

/* mbconfig.h is auto generated */
#include <libmb/mbconfig.h>

#include <libmb/mbpixbuf.h>
#include <libmb/mbexp.h>
#include <libmb/mbtray.h>
#include <libmb/mbutil.h>
#include <libmb/mbmenu.h>
#include <libmb/mbdotdesktop.h>

#undef USE_PNG
#undef USE_JPG
#undef USE_XFT
#undef USE_PANGO

#endif
